import requests
from bs4 import BeautifulSoup
import pandas as pd
import csv
import re
import time
import os
from collections import OrderedDict

INPUT_CSV = 'istmat_links.csv'
OUTPUT_CSV = 'istmat_metadata.csv'

def extract_node_id(url):
    m = re.search(r'/node/(\d+)', url)
    return m.group(1) if m else ''

def extract_title(soup):
    # Обычно заголовок — это <h1>
    h1 = soup.find('h1')
    if h1:
        return h1.get_text(strip=True)
    # fallback: <title>
    if soup.title:
        return soup.title.get_text(strip=True)
    return ''

def extract_rekvizity_block(soup):
    # Блок "Реквизиты" — ищем по тексту
    for tag in soup.find_all(['div', 'p', 'section']):
        if tag.get_text(strip=True).startswith('Реквизиты'):
            return tag
    # Альтернативно ищем по наличию слова "Реквизиты" в тексте
    for tag in soup.find_all(['div', 'p', 'section']):
        if 'Реквизиты' in tag.get_text():
            return tag
    return None

def extract_all_fields(block):
    """
    Извлекает все поля из блока "Реквизиты" вида "Название: Значение"
    Возвращает OrderedDict, чтобы сохранить порядок.
    """
    fields = OrderedDict()
    if not block:
        return fields
    text = block.get_text(separator='\n', strip=True)
    # Ищем все "Название: Значение"
    for line in text.split('\n'):
        if ':' in line:
            key, val = line.split(':', 1)
            key = key.strip().replace('(', '').replace(')', '')
            val = val.strip()
            if key and val:
                fields[key] = val
    return fields

def extract_pdf_link(soup):
    # Ищем блок "Прикреплённые файлы" и ссылки в нем
    files_block = None
    for tag in soup.find_all(['div', 'section', 'ul']):
        if 'Прикреплённые файлы' in tag.get_text():
            files_block = tag
            break
    if files_block:
        for a in files_block.find_all('a', href=True):
            if a['href'].endswith('.pdf'):
                return a['href'] if a['href'].startswith('http') else 'https://istmat.org' + a['href']
    # fallback: ищем любые pdf-ссылки
    for a in soup.find_all('a', href=True):
        if a['href'].endswith('.pdf'):
            return a['href'] if a['href'].startswith('http') else 'https://istmat.org' + a['href']
    return 'no pdf'

def parse_page(url):
    try:
        resp = requests.get(url, timeout=20)
        resp.raise_for_status()
        soup = BeautifulSoup(resp.text, 'html.parser')
        node_id = extract_node_id(url)
        title = extract_title(soup)
        rekvizity = extract_rekvizity_block(soup)
        all_fields = extract_all_fields(rekvizity)
        pdf = extract_pdf_link(soup)

        meta = OrderedDict()
        meta['sourceID'] = node_id
        meta['title'] = title
        # Добавляем все метаданные из блока "Реквизиты"
        for k, v in all_fields.items():
            meta[k] = v
        meta['pdf'] = pdf
        return meta
    except Exception as e:
        print(f'Ошибка при обработке {url}: {e}')
        return None

def main():
    if not os.path.exists(INPUT_CSV):
        print(f'Файл {INPUT_CSV} не найден!')
        return

    links = []
    with open(INPUT_CSV, encoding='utf-8') as f:
        reader = csv.DictReader(f)
        for row in reader:
            links.append(row['link'])

    processed = []
    columns = set(['id', 'sourceID', 'title', 'pdf'])  # базовые столбцы
    start_idx = 0

    if os.path.exists(OUTPUT_CSV):
        df_exist = pd.read_csv(OUTPUT_CSV)
        start_idx = len(df_exist)
        processed = df_exist.to_dict('records')
        columns.update(df_exist.columns)

    try:
        for idx, link in enumerate(links[start_idx:], start=start_idx):
            print(f'[{idx+1}/{len(links)}] Обработка: {link}')
            meta = parse_page(link)
            if meta:
                meta['id'] = f'im{idx+1:06d}'
                columns.update(meta.keys())
                processed.append(meta)
            # Сохраняем прогресс после каждой записи
            # Обеспечиваем, что все столбцы есть для всех строк
            df = pd.DataFrame(processed)
            for col in columns:
                if col not in df.columns:
                    df[col] = ''
            df = df[list(sorted(columns, key=lambda x: (x!='id', x)))]  # id первым
            df.to_csv(OUTPUT_CSV, index=False, encoding='utf-8')
            time.sleep(1)
    except Exception as e:
        print(f'Скрипт остановлен из-за ошибки: {e}')
        df = pd.DataFrame(processed)
        for col in columns:
            if col not in df.columns:
                df[col] = ''
        df = df[list(sorted(columns, key=lambda x: (x!='id', x)))]
        df.to_csv(OUTPUT_CSV, index=False, encoding='utf-8')
        print('Промежуточные данные сохранены.')

    print('Готово. Всего обработано:', len(processed))

if __name__ == '__main__':
    main()
